/*!
// @file		EmaiMessagePartEnumerator.hpp
//
// @brief		Pure virtual class, responsible for message parts parsing
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIMESSAGEPARTENUMERATOR_HPP__
#define __EMAIMESSAGEPARTENUMERATOR_HPP__

#include <EmaiMessage.hpp>

/*!
// @namespace	Emai
// @brief		Mailit for C/C++ library namespace
// @author		(C) 2006 Dacons LLP.
*/
namespace Emai
{
/*!
// @class MessagePartEnumerator
// @brief Utility class which implements basic algorithm for
//			enumerating parts in the message
*/
class MessagePartEnumerator
{
protected:

	Message								mMessage;

										MessagePartEnumerator();//no implementation
										MessagePartEnumerator(
												const MessagePartEnumerator&);//no implementation


	/*!
	// @brief	Process message header fields. Extract message sender, 
	//				recipients, subject, etc.
	//
	// @see		MessagePartEnumerator::ProcessContacts
	*/
	virtual void						ProcessHeaderFields();


	/*!
	// @brief	Process message contacts
	//
	// @param	EmaiOptions inOptions
	//
	// @see		Emai::Message::GetContactList
	// @see		EmaiMessageCopyContactList
	*/
	virtual void						ProcessContacts(
												EmaiOptions inOptions);

	/*!
	// @brief	Process message parts
	//
	// @param	MessagePart inPart
	//
	// @see		Emai::MessagePart::GetRootPart
	*/
	virtual void						ProcessMessagePart(
												MessagePart& inPart);


	/*!
	// @brief	Pure virtual function; handles the contact fields receiving
	//
	// @param	inContactList - object, which will be filled with the message
	//				contacts (according to the inOptions type specification)
	// @param	inOptions - type of the contact field
	*/
	virtual void						OnContacts(
												ContactList inContactList,
												EmaiOptions inOptions) = 0;

	/*!
	// @brief	Pure virtual function; handles the message subject field receiving
	//
	// @param	EmaiConstUniCharPtr inSubject - will be filled with the message subject
	*/
	virtual void						OnSubject(
												EmaiConstUniCharPtr inSubject) = 0;

	/*!
	// @brief	Pure virtual function; this method will be triggered,
	//				when Plain Text message part is found during message source
	//				code processing
	//				EmaiPartData::partType selector will be set to the
	//				EmaiEnumMessagePartType::EmaiPartPlainTextData value
	//
	// @param	MessagePart& inPart	- original message part
	// @param	const EmaiPartData& inPartData - parsed message part data
	//
	// @see		Emai::EmaiPartData
	// @see		Emai::EmaiEnumMessagePartType
	*/
	virtual void						OnPlainTextPart(
												MessagePart& inPart,
												const EmaiPartData& inPartData) = 0;

	/*!
	// @brief	Pure virtual function; this method will be triggered,
	//				when Unicode Plain Text message part is found during message source
	//				code processing
	//				EmaiPartData::partType selector will be set to the
	//				EmaiEnumMessagePartType::EmaiPartUnicodeTextData value
	//
	// @param	MessagePart& inPart	- original message part
	// @param	const EmaiPartData& inPartData - parsed message part data
	//
	// @see		Emai::EmaiPartData
	// @see		Emai::EmaiEnumMessagePartType
	*/
	virtual void						OnUnicodePlainTextPart(
												MessagePart& inPart,
												const EmaiPartData& inPartData) = 0;

	/*!
	// @brief	Pure virtual function; this method will be triggered,
	//				when HTML Text message part is found during message source
	//				code processing
	//				EmaiPartData::partType selector will be set to the
	//				EmaiEnumMessagePartType::EmaiPartHTMLData value
	//
	// @param	MessagePart& inPart
	// @param	const EmaiPartData& inPartData
	//
	// @see		Emai::EmaiPartData
	// @see		Emai::EmaiEnumMessagePartType
	*/
	virtual void						OnHtmlTextPart(
												MessagePart& inPart,
												const EmaiPartData& inPartData) = 0;

	/*!
	// @brief	Pure virtual function; this method will be triggered,
	//				when Data message part is found during message source
	//				code processing
	//				EmaiPartData::partType selector will be set to the
	//				EmaiEnumMessagePartType::EmaiPartDataData value
	//
	// @param	MessagePart& inPart
	// @param	const EmaiPartData& inPartData
	//
	// @see		Emai::EmaiPartData
	// @see		Emai::EmaiEnumMessagePartType
	*/
	virtual void						OnDataPart(
												MessagePart& inPart,
												const EmaiPartData& inPartData) = 0;

	/*!
	// @brief	Pure virtual function; this method will be triggered,
	//				when Composite message part start is found during message source
	//				code processing. For example, when Multipart/Mixed block begins.
	//				EmaiPartData::partType selector will be set to the
	//				EmaiEnumMessagePartType::EmaiPartCompositeData value
	//
	// @param	MessagePart& inPart
	// @param	const EmaiPartData& inPartData
	//
	// @see		Emai::EmaiPartData
	// @see		Emai::EmaiEnumMessagePartType
	*/
	virtual void						OnCompositePartStart(
												MessagePart& inPart,
												const EmaiPartData& inPartData) = 0;

	/*!
	// @brief	Pure virtual function; this method will be triggered,
	//				when Composite message part end is found during message source
	//				code processing. For example, when Multipart/Mixed block ends.
	//				EmaiPartData::partType selector will be set to the
	//				EmaiEnumMessagePartType::EmaiPartCompositeData value
	//
	// @param	MessagePart& inPart
	// @param	const EmaiPartData& inPartData
	//
	// @see		Emai::EmaiPartData
	// @see		Emai::EmaiEnumMessagePartType
	*/
	virtual void						OnCompositePartEnd(
												MessagePart& inPart,
												const EmaiPartData& inPartData) = 0;


	/*!
	// @brief	Processes all of the message parts, according to the part tree
	*/
	virtual void						ProcessMessageParts();

	/*!
	// @brief	Called whenever message parts are about to be enumerated.
	//				Default implementation doesn nothing
	*/
	virtual void						PreProcessMessageParts();

	/*!
	// @brief	Called whenever message parts are enumerated.
	//				Default implementation doesn nothing
	*/
	virtual void						PostProcessMessageParts();

public:
	virtual								~MessagePartEnumerator();
	
	//constructors
										MessagePartEnumerator(
												const Message& inMessage);
										
	/*!
	// @brief	Starts message enumerating
	*/
	void								Enumerate();
};


/*
// EmaiMessagePart implemetation
*/
//constructors
inline
MessagePartEnumerator::MessagePartEnumerator(
		const Message& inMessage
	):
	mMessage(inMessage)
{
}

inline
MessagePartEnumerator::~MessagePartEnumerator()
{
}

inline void
MessagePartEnumerator::Enumerate()
{
	ProcessHeaderFields();
	ProcessMessageParts();
}

inline void
MessagePartEnumerator::ProcessHeaderFields()
{
	ProcessContacts(EmaiFrom);
	ProcessContacts(EmaiTo);
	ProcessContacts(EmaiCC);
	ProcessContacts(EmaiSender);
	ProcessContacts(EmaiReplyTo);
	ProcessContacts(EmaiBCC);
	EmaiConstUniCharPtr subject = mMessage.GetField(NULL, EmaiSubject);
	if (subject != NULL)
		OnSubject(subject);
}

inline void
MessagePartEnumerator::ProcessContacts(
		EmaiOptions inOptions
	)
{
	Emai::ContactList contactList;
	try
	{
		contactList = mMessage.CopyContactList(inOptions);
	}
	catch (Emai::Exception& /*exception*/)
	{
		//contact list cannot be parsed: just use noone@nowhere
		EmaiUniChar dummyAddress[] = {'n', 'o', 'o', 'n', 'e', '@', 'n', 'o', 'w', 'h', 'e', 'r', 'e', '\0'};
		contactList.AddContact(mMessage.GetField(NULL, inOptions), dummyAddress);
	}
	if (contactList != NULL)
		OnContacts(contactList, inOptions);
}


inline void
MessagePartEnumerator::ProcessMessageParts()
{
	PreProcessMessageParts();
	MessagePart rootPart(mMessage.GetRootPart());
	ProcessMessagePart(rootPart);
	PostProcessMessageParts();
}

inline void
MessagePartEnumerator::PreProcessMessageParts()
{
}

inline void
MessagePartEnumerator::PostProcessMessageParts()
{
}

inline void
MessagePartEnumerator::ProcessMessagePart(
		MessagePart& inPart
	)
{
	EmaiPartData dataDesc = {EmaiStructureVersionFirst};
	try
	{
		inPart.GetData(dataDesc);
		switch (dataDesc.partType)
		{
			case EmaiPartPlainTextData:
				OnPlainTextPart(inPart, dataDesc);
				break;
			case EmaiPartUnicodeTextData:
				OnUnicodePlainTextPart(inPart, dataDesc);
				break;
			case EmaiPartHTMLData:
				OnHtmlTextPart(inPart, dataDesc);
				break;
			case EmaiPartDataData:
				OnDataPart(inPart, dataDesc);
				break;
			case EmaiPartCompositeData:
			{
				OnCompositePartStart(inPart, dataDesc);

				for (EmaiUint32 i = 0; i < dataDesc.partData.composite.subPartCount;
					++i)
				{
					MessagePart subPart(dataDesc.partData.composite.subParts[i], true);
					ProcessMessagePart(subPart);
				}

				OnCompositePartEnd(inPart, dataDesc);

				break;
			}
			default:
				break;
		}
	}
	catch (Emai::Exception& /*exception*/)
	{
	}
}


}
#endif /*__EMAIMESSAGEPARTENUMERATOR_HPP__*/
